@@ -6,12 +6,13 @@ import sys
 from collections import defaultdict
 from dataclasses import dataclass, field
 from io import IOBase
-from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO
-from urllib.parse import urlencode
+from typing import Union, List, Optional, Dict, DefaultDict, Tuple, BinaryIO, Any
+from urllib.parse import urlencode, urljoin
 
 from requests.auth import HTTPBasicAuth, HTTPDigestAuth, AuthBase
+from requests.structures import CaseInsensitiveDict
 
-from .utils import get_real_file_path, triple_or_double_tostring
+from .utils import get_real_file_path, triple_or_double_tostring, APPLICATION_JSON, json_to_urlencoded_array
 
 try:
     import jstyleson as json
@@ -25,7 +26,7 @@ from textx import TextXSyntaxError, metamodel_from_file
 from .dsl_jsonparser import json_or_array_to_json
 from .exceptions import *
 from .parse_models import Allhttp, AuthWrap, DigestAuth, BasicAuth, Line, Query, Http, NameWrap, UrlWrap, Header, \
-    MultiPartFile, FilesWrap, TripleOrDouble, Payload as ParsePayload
+    MultiPartFile, FilesWrap, TripleOrDouble, Payload as ParsePayload, Certificate, P12Certificate, ExtraArg
 from .property_schema import property_schema
 from .property_util import PropertyProvider
 
@@ -68,7 +69,7 @@ class Config:
 
 @dataclass
 class Payload:
-    data: Optional[Union[str, bytes, Dict]] = None
+    data: Optional[Union[str, bytes, Dict, BinaryIO]] = None
     json: Optional[Dict] = None
     header: Optional[str] = None
     filename: str = None
@@ -88,7 +89,11 @@ class HttpDef:
     query: dict = None
     auth: AuthBase = None
     payload: Optional[Payload] = None
+    certificate: Optional[List[str]] = None
+    p12: Optional[List[str]] = None
     output: str = None
+    allow_insecure = False
+    session_clear = False
     test_script: str = ""
 
     def get_har(self):
@@ -105,15 +110,19 @@ class HttpDef:
         if not self.payload:
             return None
         payload = self.payload
-        return_data = {"mimeType": self.payload.header}
+        return_data = {}
         if payload.data:
             if isinstance(payload.data, dict):
-                return_data["text"] = urlencode(payload.data)
+                return_data["mimeType"] = FORM_URLENCODED
+                return_data["text"] = urlencode(json_to_urlencoded_array(payload.data))
             else:
+                return_data["mimeType"] = payload.header or "text/plain"
                 return_data["text"] = payload.data
         elif payload.json:
+            return_data["mimeType"] = APPLICATION_JSON
             return_data["text"] = json.dumps(payload.json)
         elif payload.files:
+            return_data["mimeType"] = MULTIPART_FORM_INPUT
             params = []
             for (name, (multipart_filename, multipart_content, mimetype)) in payload.files:
                 content = multipart_content
@@ -143,47 +152,62 @@ class HttpDef:
         json_payload = None
         fileswrap = None
         type = None
-        if self.payload.filename:
-            file = self.payload.filename
-        elif isinstance(self.payload.data, str):
-            data = [TripleOrDouble(str=self.payload.data)]
-        elif isinstance(self.payload.data, dict):
-            datajson = None
-        elif self.payload.json:
-            json_payload = self.payload.json
-        elif self.payload.files:
-            fileswrap = FilesWrap([])
-            for filekey, multipartdata in self.payload.files:
-                fileswrap.files.append(
-                    MultiPartFile(name=filekey,
-                                  path=multipartdata[1].name if multipartdata[0] else multipartdata[1],
-                                  type=multipartdata[2] if len(multipartdata) > 2 else None)
-                )
+        payload = None
+        if self.payload:
+            if self.payload.filename:
+                file = self.payload.filename
+            elif isinstance(self.payload.data, str):
+                data = [TripleOrDouble(str=self.payload.data)]
+            elif isinstance(self.payload.data, dict):
+                datajson = self.payload.data
+            elif self.payload.json:
+                json_payload = self.payload.json
+            elif self.payload.files:
+                fileswrap = FilesWrap([])
+                for filekey, multipartdata in self.payload.files:
+                    fileswrap.files.append(
+                        MultiPartFile(name=filekey,
+                                      path=multipartdata[1].name if multipartdata[0] else multipartdata[1],
+                                      type=multipartdata[2] if len(multipartdata) > 2 else None)
+                    )
 
-        payload = ParsePayload(data=data, datajson=datajson, file=file, json=json_payload, fileswrap=fileswrap,
-                               type=type)
+            payload = ParsePayload(data=data, datajson=datajson, file=file, json=json_payload, fileswrap=fileswrap,
+                                   type=type)
 
         query_lines = []
-        for key, values in self.query.items():
-            for value in values:
-                query_lines.append(Line(header=None, query=Query(key=key, value=value)))
+        if self.query:
+            for key, values in self.query.items():
+                for value in values:
+                    query_lines.append(Line(header=None, query=Query(key=key, value=value)))
         auth_wrap = None
         if self.auth:
             if isinstance(self.auth, HTTPBasicAuth):
                 auth_wrap = AuthWrap(basic_auth=BasicAuth(self.auth.username, self.auth.password))
             elif isinstance(self.auth, HTTPDigestAuth):
                 auth_wrap = AuthWrap(digest_auth=DigestAuth(self.auth.username, self.auth.password))
-        return Allhttp(allhttps=[Http(
+        certificate = None
+        if self.certificate:
+            certificate = Certificate(*self.certificate)
+        elif self.p12:
+            certificate = P12Certificate(*self.p12)
+        extra_args = []
+        if self.session_clear:
+            extra_args.append(ExtraArg(clear="@clear"))
+        if self.allow_insecure:
+            extra_args.append(ExtraArg(insecure="@insecure"))
+        header_lines = []
+        if self.headers:
+            for key, value in self.headers.items():
+                header_lines.append(Line(header=Header(key=key, value=value), query=None))
+        return Http(
             namewrap=NameWrap(self.name),
+            extra_args=extra_args,
             urlwrap=UrlWrap(url=self.url, method=self.method),
-            lines=[
-                      Line(header=Header(key=key, value=value), query=None)
-                      for key, value in
-                      self.headers.items()] + query_lines
-            ,
+            lines=header_lines + query_lines,
             payload=payload,
-            output=None, authwrap=auth_wrap
-        )])
+            certificate=certificate,
+            output=None, authwrap=auth_wrap, description=None
+        )
 
 
 @dataclass
@@ -314,7 +338,7 @@ class BaseModelProcessor:
     def load_content(self):
         if not os.path.exists(self.file):
             raise HttpFileNotFoundException(file=self.file)
-        with open(self.file, 'r') as f:
+        with open(self.file, 'r', encoding="utf-8") as f:
             self.original_content = self.content = f.read()
 
     def get_updated_content(self, content) -> str:
@@ -325,36 +349,41 @@ class BaseModelProcessor:
 
     def select_target(self):
         if target := self.args.target:
-            if not isinstance(target, str):
-                target = str(target)
-            if target.isdecimal():
-                if 1 <= int(target) <= len(self.model.allhttps):
-                    self.http = self.model.allhttps[int(target) - 1]
-                else:
-                    raise ParameterException(message="target startswith 1", key='target',
-                                             value=target)
-            else:
-                try:
-                    # if multiple names have same value, it will create confusion
-                    # if they want to go with that. then pass id
-                    self.http = next(filter(lambda http: http.namewrap.name == target,
-                                            (http for http in self.model.allhttps if http.namewrap)))
-                except StopIteration:
-                    raise ParameterException(message="target is not spelled correctly", key='target',
-                                             value=target)
+            self.http = self.get_target(target, self.model.allhttps)
         else:
             self.http = self.model.allhttps[0]
         self.base_http = None
         if self.http.namewrap and self.http.namewrap.base:
             base = self.http.namewrap.base
-            self.base_http = None
-            for http in self.model.allhttps:
-                if http.namewrap and http.namewrap.name == base:
-                    self.base_http = http
-                    break
-            if not self.base_http:
+            if base == self.http.namewrap.name:
+                raise ParameterException(message="target and base should not be equal", key=target,
+                                         value=base)
+            try:
+                self.base_http = self.get_target(base, self.model.allhttps)
+            except Exception:
                 raise UndefinedHttpToExtend(target=self.http.namewrap.name, base=base)
 
+    @staticmethod
+    def get_target(target: Union[str, int], http_def_list: List[Http]):
+        if not isinstance(target, str):
+            target = str(target)
+        if target.isdecimal():
+            if 1 <= int(target) <= len(http_def_list):
+                selected = http_def_list[int(target) - 1]
+            else:
+                raise ParameterException(message="target startswith 1", key='target',
+                                         value=target)
+        else:
+            try:
+                # if multiple names have same value, it will create confusion
+                # if they want to go with that. then pass id
+                selected = next(filter(lambda http: http.namewrap.name == target,
+                                       (http for http in http_def_list if http.namewrap)))
+            except StopIteration:
+                raise ParameterException(message="target is not spelled correctly", key='target',
+                                         value=target)
+        return selected
+
     def validate_names(self):
         names = []
         for index, http in enumerate(self.model.allhttps):
@@ -397,7 +426,9 @@ class HttpDefBase(BaseModelProcessor):
                 4. dev can define in data/file/files's type attribute section for ('content-type')
         :return:
         """
-        headers = {}
+        ## headers are case insensitive
+        ## having duplicate headers creates problem while exporting to curl,postman import..
+        headers = CaseInsensitiveDict()
         headers.update(self.default_headers)
         self.load_headers_to_dict(self.base_http, headers)
         self.load_headers_to_dict(self.http, headers)
@@ -414,10 +445,40 @@ class HttpDefBase(BaseModelProcessor):
                 self.remove_quotes(header, '"')
                 headers[self.get_updated_content(header.key)] = self.get_updated_content(header.value)
 
+    def load_certificate(self):
+        request_logger.debug(
+            f'url is {self.http.certificate}')
+        certificate: Union[Certificate, P12Certificate] = self.get_current_or_base("certificate")
+        if certificate:
+            if certificate.cert:
+                self.httpdef.certificate = [self.get_updated_content(certificate.cert),
+                                            self.get_updated_content(certificate.key) if certificate.key else None]
+            elif certificate.p12_file:
+                self.httpdef.p12 = [self.get_updated_content(certificate.p12_file),
+                                    self.get_updated_content(certificate.password)]
+
+    def load_extra_flags(self):
+        # flags are extendable
+        # once its marked as allow insecure
+        # user would want all child to have same effect
+        extra_args = self.http.extra_args
+        if self.base_http and self.base_http.extra_args:
+            extra_args += self.base_http.extra_args
+        if extra_args:
+            for flag in extra_args:
+                if flag.clear:
+                    self.httpdef.session_clear = True
+                elif flag.insecure:
+                    self.httpdef.allow_insecure = True
+
     def load_url(self):
         request_logger.debug(
             f'url is {self.http.urlwrap.url}')
-        self.httpdef.url = self.get_updated_content(self.http.urlwrap.url)
+        if base_http := self.base_http:
+            self.httpdef.url = urljoin(self.get_updated_content(base_http.urlwrap.url),
+                                       self.get_updated_content(self.http.urlwrap.url))
+        else:
+            self.httpdef.url = self.get_updated_content(self.http.urlwrap.url)
 
     def load_method(self):
         if method := self.http.urlwrap.method:
@@ -464,16 +525,7 @@ class HttpDefBase(BaseModelProcessor):
             # varstring hanlding
             return Payload(data=d, header=FORM_URLENCODED)
         elif upload_filename := self.http.payload.file:
-            upload_filename = self.get_updated_content(upload_filename)
-            request_logger.debug(
-                f'payload will be loaded from `{upload_filename}`')
-            if not os.path.exists(upload_filename):
-                request_logger.debug(
-                    f'payload file `{upload_filename}` Not found. ')
-                raise DataFileNotFoundException(datafile=upload_filename)
-            mimetype = self.get_mimetype_from_file(upload_filename, self.http.payload.type)
-            with open(upload_filename, 'rb') as f:
-                return Payload(data=f.read(), header=mimetype, filename=upload_filename)
+            return self.load_payload_fileinput(upload_filename)
         elif json_data := self.http.payload.json:
             d = json_or_array_to_json(json_data, self.get_updated_content)
             return Payload(json=d, header=MIME_TYPE_JSON)
@@ -494,6 +546,18 @@ class HttpDefBase(BaseModelProcessor):
             return Payload(files=files, header=MULTIPART_FORM_INPUT)
         return Payload()
 
+    def load_payload_fileinput(self, upload_filename):
+        upload_filename = self.get_updated_content(upload_filename)
+        request_logger.debug(
+            f'payload will be loaded from `{upload_filename}`')
+        if not os.path.exists(upload_filename):
+            request_logger.debug(
+                f'payload file `{upload_filename}` Not found. ')
+            raise DataFileNotFoundException(datafile=upload_filename)
+        mimetype = self.get_mimetype_from_file(upload_filename, self.http.payload.type)
+        f = open(upload_filename, 'rb')
+        return Payload(data=f, header=mimetype, filename=upload_filename)
+
     @staticmethod
     def get_mimetype_from_file(filename, mimetype: Optional[str]) -> Optional[str]:
         if mimetype:
@@ -530,11 +594,7 @@ class HttpDefBase(BaseModelProcessor):
             return sys.stdout
 
     def load_auth(self):
-        auth_wrap = None
-        if self.http.authwrap:
-            auth_wrap = self.http.authwrap
-        elif self.base_http:
-            auth_wrap = self.base_http.authwrap
+        auth_wrap = self.get_current_or_base("authwrap")
         if auth_wrap:
             if basic_auth := auth_wrap.basic_auth:
                 self.httpdef.auth = HTTPBasicAuth(self.get_updated_content(basic_auth.username),
@@ -545,6 +605,12 @@ class HttpDefBase(BaseModelProcessor):
                                                    self.get_updated_content(
                                                        digest_auth.password))
 
+    def get_current_or_base(self, attr_key) -> Any:
+        if getattr(self.http, attr_key):
+            return getattr(self.http, attr_key)
+        elif self.base_http:
+            return getattr(self.base_http, attr_key)
+
     def load_def(self):
         if self._loaded:
             return
@@ -555,7 +621,9 @@ class HttpDefBase(BaseModelProcessor):
         self.load_query()
         self.load_payload()
         self.load_auth()
+        self.load_certificate()
         self.load_test_script()
+        self.load_extra_flags()
         self._loaded = True
 
     def load_test_script(self):
