@@ -12,11 +12,13 @@ from requests.status_codes import _codes as status_code
 from requests_pkcs12 import Pkcs12Adapter
 from textx import metamodel_from_file
 
+from dothttp import APPLICATION_JSON
 from . import eprint, Config, HttpDefBase, js3py
 from .curl_utils import to_curl
 from .dsl_jsonparser import json_or_array_to_json
 from .json_utils import JSONEncoder
 from .parse_models import Allhttp
+from .utils import quote_or_unquote
 
 JSON_ENCODER = JSONEncoder(indent=4)
 
@@ -160,25 +162,31 @@ class CurlCompiler(RequestBase):
             parts.append(("--cert-type", f"P12"))
         if self.httpdef.allow_insecure:
             parts.append(("-k", None))
+        payload_parts = []
         if self.http.payload:
             if self.http.payload.file:
-                parts.append(('--data', "@" + self.get_updated_content(self.http.payload.file)))
+                payload_parts += [('--data', "@" + self.get_updated_content(self.http.payload.file))]
             elif self.http.payload.fileswrap:
                 if payload.files:
                     for file in payload.files:
                         if isinstance(file[1][1], str):
-                            parts.append(('--form', file[0] + "=" + file[1][1]))
+                            payload_parts.append(('--form', file[0] + "=" + file[1][1]))
                         else:
-                            parts.append(('--form', file[0] + "=@" + file[1][1].name))
+                            payload_parts.append(('--form', file[0] + "=@" + file[1][1].name))
             elif self.http.payload.json:
                 dumps = json.dumps(payload.json, indent=4)
-                prep.headers["content-type"] = "application/json"
-                prep.prepare_body(data=dumps, files=None)
-            else:
+                self.httpdef.headers['content-type'] = APPLICATION_JSON
+                payload_parts += [('-d', dumps)]
+            elif self.http.payload.data:
                 if payload.header:
-                    prep.headers['content-type'] = payload.header
-                prep.prepare_body(data=payload.data, json=payload.json, files=payload.files)
-        curl_req = to_curl(prep, parts)
+                    self.httpdef.headers['content-type'] = payload.header
+                payload_parts += [('-d', payload.data)]
+        # there few headers set dynamically
+        # so set headers in the end
+        for k, v in sorted(self.httpdef.headers.items()):
+            parts += [('-H', '{0}: {1}'.format(k, v))]
+        parts += payload_parts
+        curl_req = to_curl(self.httpdef, parts)
         return curl_req
 
 
@@ -197,8 +205,13 @@ class HttpFileFormatter(RequestBase):
         output_str = ""
         for http in model.allhttps:
             new_line = "\n"
-            if namewrap := http.namewrap:
-                output_str += f"@name(\"{namewrap.name}\"){new_line}"
+            if getattr(http, "description", None):
+                for line in http.description.splitlines():
+                    output_str += "// " + line + new_line
+                output_str += new_line
+            if http.namewrap and http.namewrap.name:
+                quote_type, name = quote_or_unquote(http.namewrap.name)
+                output_str += f"@name({quote_type}{name}{quote_type}){new_line}"
             method = http.urlwrap.method if http.urlwrap.method else "GET"
             output_str += f'{method} "{http.urlwrap.url}"'
             if auth_wrap := http.authwrap:
@@ -207,12 +220,15 @@ class HttpFileFormatter(RequestBase):
                 elif digest_auth := auth_wrap.digest_auth:
                     output_str += f'{new_line}digestauth("{digest_auth.username}", "{digest_auth.password}")'
             if lines := http.lines:
-                headers = new_line.join(map(lambda line: f'"{line.header.key}": "{line.header.value}"',
-                                            filter(lambda line:
-                                                   line.header, lines)))
+                def check_for_quotes(line):
+                    quote_type, value = quote_or_unquote(line.header.value)
+                    return f'"{line.header.key}": {quote_type}{value}{quote_type}'
+
+                headers = new_line.join(map(check_for_quotes, filter(lambda line: line.header, lines)))
                 if headers:
                     output_str += f"\n{headers}"
-                query = new_line.join(map(lambda line: f'? "{line.query.key}"= "{line.query.value}"',
+
+                query = new_line.join(map(query_to_http,
                                           filter(lambda line:
                                                  line.query, lines)))
                 if query:
@@ -260,6 +276,11 @@ class HttpFileFormatter(RequestBase):
                 f.write(formatted)
 
 
+def query_to_http(line):
+    quote_type = quote_or_unquote(line.query.value)[0]
+    return f'? {quote_type}{line.query.key}{quote_type}= {quote_type}{line.query.value}{quote_type}'
+
+
 class RequestCompiler(RequestBase):
 
     def run(self):
